@@ -47,6 +47,8 @@ CDECL const char *strerror(int);
 	       && defined(HAVE_GETNAMEINFO) \
 	       && defined(HAVE_GETADDRINFO))
 
+#define USE_EXPAT (defined(HAVE_EXPAT_H) && defined(HAVE_LIBEXPAT))
+
 #if defined(SOCKS4) || defined(SOCKS_DANTE)
 # define connect     Rconnect
 # define getsockname Rgetsockname
@@ -80,6 +82,7 @@ CDECL void SOCKSinit(const char *);
 #ifdef SOCKS5
 # define SOCKS
 # include <socks.h>
+# undef getc /* fix a problem with redefined getc */
 #endif
 
 #include <sys/types.h>
@@ -109,6 +112,11 @@ CDECL void SOCKSinit(const char *);
 CDECL char *strtok_r(char *str, const char *sep, char **lasts);
 #endif
 
+#if !HAVE_DECL_MEMMEM
+CDECL void *memmem(const void *haystack, size_t haystack_len,
+		   const void *needle, size_t needle_len);
+#endif
+
 /* Tell the compiler when a conditional or integer expression is
    almost always true or almost always false.  */
 #ifndef HAVE_BUILTIN_EXPECT
@@ -139,4 +147,6 @@ CDECL char *strtok_r(char *str, const char *sep, char **lasts);
 # include "trio.h"
 #endif
 
+#define USE_SSL (defined(USE_GNUTLS) || defined(USE_OPENSSL))
+
 #endif /* POST_CONFIG_H */
