@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.102 2003/11/14 09:20:00 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.108 2004/05/11 07:31:48 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -206,6 +206,7 @@ restart:
       }
 
       args->setarg(0,c->name); // in case it was abbreviated
+      args->rewind();
 
       if(cmd==0)
 	 cmd=args->Combine();
@@ -272,22 +273,20 @@ void CmdExec::SuspendJob(Job *j)
 
 void CmdExec::ExecParsed(ArgV *a,FDStream *o,bool b)
 {
-   bool old_running=running;
-   running=true; // disable reentering
+   Enter();
 
    if(args)
       delete args;
    args=a;
    xfree(cmd);
    cmd=args->Combine();
-   if(output)
-      delete output;
+   delete output;
    output=o;
    background=b;
    condition=COND_ANY;
    exec_parsed_command();
 
-   running=old_running;
+   Leave();
 }
 
 bool CmdExec::Idle()
@@ -387,8 +386,7 @@ int CmdExec::Do()
 	       // cd to another url.
 	       const char *loc_c=session->GetNewLocation();
 	       int max_redirections=ResMgr::Query("xfer:max-redirections",0);
-	       if(loc_c && loc_c[0] && max_redirections>0
-	       && loc_c[strlen(loc_c)-1]=='/')
+	       if(loc_c && max_redirections>0 && last_char(loc_c)=='/')
 	       {
 		  eprintf(_("%s: received redirection to `%s'\n"),"cd",loc_c);
 		  if(++redirections>max_redirections)
@@ -540,7 +538,7 @@ int CmdExec::Do()
 	    return MOVED;
 	 }
       }
-      if(status_line)
+      if(status_line && status_line->CanShowNow())
 	 ShowRunStatus(status_line);   // this is only for top level CmdExec.
       return m;
    }
@@ -584,7 +582,7 @@ int CmdExec::Do()
 	    return MOVED;
 	 }
       }
-      if(status_line)
+      if(status_line && status_line->CanShowNow())
 	 ShowRunStatus(status_line);   // this is only for top level CmdExec.
       return m;
    }
@@ -624,7 +622,7 @@ try_get_cmd:
 	 {
 	    if(next_cmd && *next_cmd && partial_cmd)
 	    {
-	       if(next_cmd[strlen(next_cmd)-1]!='\n')
+	       if(last_char(next_cmd)!='\n')
 	       {
 		  // missing EOL on last line, add it
 		  FeedCmd("\n");
@@ -1104,11 +1102,12 @@ char *ArgV::CombineQuoted(int start) const
 {
    int	 i;
    char  *res;
-   char	 *store,*arg;
+   char	 *store;
+   const char *arg;
    int	 len=0;
 
-   for(i=start; i<c; i++)
-      len+=strlen(v[i])*2+3;
+   for(i=start; i<Count(); i++)
+      len+=strlen(String(i))*2+3;
 
    if(len==0)
       return(xstrdup(""));
@@ -1116,9 +1115,9 @@ char *ArgV::CombineQuoted(int start) const
    res=(char*)xmalloc(len);
 
    store=res;
-   for(i=start; i<c; i++)
+   for(i=start; i<Count(); i++)
    {
-      arg=v[i];
+      arg=String(i);
       if(CmdExec::needs_quotation(arg))
       {
 	 *store++='"';
