@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.106.2.1 2003/12/17 09:52:49 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.120 2005/04/04 09:59:38 lav Exp $ */
 
 #include <config.h>
 
@@ -27,9 +27,6 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#ifdef HAVE_NETINET_TCP_H
-# include <netinet/tcp.h>
-#endif
 #include <errno.h>
 #include <ctype.h>
 #include <assert.h>
@@ -69,7 +66,7 @@ void FileAccess::Init()
    mode=CLOSED;
    try_time=0;
    retries=0;
-   time(&event_time);
+   event_time=now;
    opt_date=0;
    opt_size=0;
    error=0;
@@ -80,6 +77,8 @@ void FileAccess::Init()
    norest_manual=false;
    location=0;
    suggested_filename=0;
+   entity_content_type=0;
+   entity_charset=0;
    array_ptr=array_cnt=0;
 
    url=0;
@@ -134,6 +133,8 @@ FileAccess::~FileAccess()
    xfree(url);
    xfree(closure);
    xfree(location);
+   xfree(entity_content_type);
+   xfree(entity_charset);
    for(FileAccess **scan=&chain; *scan; scan=&((*scan)->next))
    {
       if(*scan==this)
@@ -148,40 +149,14 @@ void  FileAccess::DebugPrint(const char *prefix,const char *str,int level)
 {
    if(!Log::global)
       return;
-
-   char *msg=(char*)alloca(xstrlen(hostname)+strlen(prefix)+strlen(str)+25);
-
-   do
-   {
-      if(Log::global->IsTTY())
-	 strcpy(msg,prefix);
-      else
-      {
-	 sprintf(msg,"[%d] ",(int)getpid());
-	 if(hostname)
-	 {
-	    strcat(msg,hostname);
-	    strcat(msg," ");
-	 }
-	 strcat(msg,prefix);
-      }
-
-      int msglen=strlen(msg);
-      while(*str && *str!='\n')
-      {
-	 if(!(*str=='\r' && (str[1]=='\n' || str[1]=='\0')))
-	    msg[msglen++]=*str;
-	 str++;
-      }
-      msg[msglen++]='\n';
-      msg[msglen]=0;
-
-      Log::global->Write(level,msg);
-
-      if(!*str++)
-         break;
-   }
-   while(*str);
+   int len=strlen(str);
+   if(len>0 && str[len-1]=='\n')
+      len--;
+   if(len>0 && str[len-1]=='\r')
+      len--;
+   char *buf=string_alloca(strlen(prefix)+len+2);
+   sprintf(buf,"%s%.*s\n",prefix,len,str);
+   Log::global->Write(level,buf);
 }
 
 int   FileAccess::Poll(int fd,int ev)
@@ -256,6 +231,7 @@ void  FileAccess::Open(const char *fn,int mode,off_t offs)
    if(IsOpen())
       Close();
    Resume();
+   event_time=now;
    file=xstrdup(fn);
    real_pos=-1;
    pos=offs;
@@ -336,6 +312,11 @@ const char *FileAccess::StrError(int err)
       }
       return(_("Login failed"));
    case(NOT_SUPP):
+      if(error)
+      {
+   	 sprintf(str,"%s: %s",_("Operation not supported"),error);
+	 return(str);
+      }
       return(_("Operation not supported"));
    case(FILE_MOVED):
       if(error)
@@ -368,6 +349,10 @@ void FileAccess::Close()
    ascii=false;
    norest_manual=false;
    xfree(location); location=0;
+   xfree(entity_content_type);
+   entity_content_type=0;
+   xfree(entity_charset);
+   entity_charset=0;
    ClearError();
 }
 
@@ -466,12 +451,9 @@ void FileAccess::Login(const char *user1,const char *pass1)
       }
       if(pass==0 && hostname) // still no pass? Try .netrc
       {
-	 NetRC::Entry *nrc=NetRC::LookupHost(hostname);
+	 NetRC::Entry *nrc=NetRC::LookupHost(hostname,user);
 	 if(nrc)
-	 {
-	    if(nrc->user && !strcmp(nrc->user,user))
-	       pass=xstrdup(nrc->pass);
-	 }
+	    pass=xstrdup(nrc->pass);
       }
    }
    ResetLocationData();
@@ -632,8 +614,8 @@ void FileAccess::OptimizePath(char *path)
    {
       if(in[0]=='/')
       {
-	 // double slash or a slash on the end
-	 if(in[1]=='/' || in[1]=='\0')
+	 // double slash
+	 if(in[1]=='/') // || (strip_trailing_slash && in[1]=='\0'))
 	 {
 	    in++;
 	    continue;
@@ -695,7 +677,7 @@ void FileAccess::Chdir(const char *path,bool verify)
    {
       if(cwd && cwd[0])
       {
-	 if(cwd[strlen(cwd)-1]=='/')
+	 if(last_char(cwd)=='/')
 	    sprintf(newcwd,"%s%s",cwd,path);
 	 else
 	    sprintf(newcwd,"%s/%s",cwd,path);
@@ -705,6 +687,7 @@ void FileAccess::Chdir(const char *path,bool verify)
    }
 
    OptimizePath(newcwd);
+   strip_trailing_slashes(newcwd);
 
    if(verify)
       Open(newcwd,CHANGE_DIR);
@@ -931,7 +914,7 @@ DirList *FileAccess::MakeDirList(ArgV *a) { if(a) delete a; return 0; }
 
 DirList::~DirList()
 {
-   Delete(buf);
+   delete buf;
    if(args)
       delete args;
 }
@@ -939,7 +922,11 @@ DirList::~DirList()
 void FileAccess::CleanupAll()
 {
    for(FileAccess *o=chain; o!=0; o=o->next)
+   {
+      Enter(o);
       o->CleanupThis();
+      Leave(o);
+   }
 }
 
 FileAccess *FileAccess::NextSameSite(FA *scan)
@@ -1060,6 +1047,12 @@ void FileAccessOperation::SetError(const char *e)
    error_text=xstrdup(e);
    done=true;
 }
+void FileAccessOperation::SetErrorCached(const char *e)
+{
+   xfree(error_text);
+   error_text=xasprintf(_("%s [cached]"),e);
+   done=true;
+}
 
 
 // ListInfo implementation
@@ -1124,10 +1117,17 @@ void ListInfo::SetExclude(const char *p,PatternSet *x)
 #else
 # define _fish
 #endif
+#ifndef MODULE_PROTO_SFTP
+# include "SFtp.h"
+# define _sftp SFtp::ClassInit()
+#else
+# define _sftp
+#endif
 void FileAccess::ClassInit()
 {
    _ftp;
    _file;
    _http;
    _fish;
+   _sftp;
 }
