@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.106 2004/10/20 14:20:54 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.108 2005/04/04 09:59:39 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -46,7 +46,6 @@
 #include "misc.h"
 #include "LsCache.h"
 #include "plural.h"
-#include "OutputJob.h"
 
 #define skip_threshold 0x1000
 #define debug(a) Log::global->Format a
@@ -892,8 +891,14 @@ void FileCopyPeerFA::OpenSession()
       }
       const char *b;
       int s;
-      if(use_cache && LsCache::Find(session,file,FAmode,&b,&s))
+      int err;
+      if(use_cache && LsCache::Find(session,file,FAmode,&err,&b,&s))
       {
+	 if(err)
+	 {
+	    SetError(b);
+	    return;
+	 }
 	 size=s;
 	 if(seek_pos>=s)
 	    goto past_eof;
@@ -1065,7 +1070,7 @@ int FileCopyPeerFA::Get_LL(int len)
    if(res==0)
    {
       eof=true;
-      LsCache::Add(session,file,FAmode,this);
+      LsCache::Add(session,file,FAmode,FA::OK,this);
    }
    return res;
 }
@@ -1692,77 +1697,5 @@ int FileCopyPeerDirList::Do()
    return MOVED;
 }
 
-FileCopyPeerOutputJob::FileCopyPeerOutputJob(OutputJob *new_o)
-   : FileCopyPeer(PUT)
-{
-   o=new_o;
-   DontCopyDate();
-}
-
-int FileCopyPeerOutputJob::Put_LL(const char *buf,int len)
-{
-   off_t io_at=pos;
-   if(GetRealPos()!=io_at) // GetRealPos can alter pos.
-      return 0;
-
-   if(len==0 && eof)
-      return 0;
-
-   if(o->Full())
-      return 0;
-
-   o->Put(buf,len);
-
-   seek_pos+=len; // mainly to indicate that there was some output.
-   return len;
-}
-
-int FileCopyPeerOutputJob::Do()
-{
-   if(o->Error())
-   {
-      broken=true;
-      return MOVED;
-   }
-
-   if(eof && !in_buffer)
-   {
-      done=true;
-      return MOVED;
-   }
-
-   int m=STALL;
-
-   if(!write_allowed)
-      return m;
-
-   while(in_buffer>0)
-   {
-      int res=Put_LL(buffer+buffer_ptr,in_buffer);
-      if(res>0)
-      {
-	 in_buffer-=res;
-	 buffer_ptr+=res;
-	 m=MOVED;
-      }
-      if(res<0)
-	 return MOVED;
-      if(res==0)
-	 break;
-   }
-   return m;
-}
-
-void FileCopyPeerOutputJob::Fg()
-{
-   o->Fg();
-   FileCopyPeer::Fg();
-}
-void FileCopyPeerOutputJob::Bg()
-{
-   o->Bg();
-   FileCopyPeer::Bg();
-}
-
 // special pointer to creator of ftp/ftp copier. It is init'ed in Ftp class.
 FileCopy *(*FileCopy::fxp_create)(FileCopyPeer *src,FileCopyPeer *dst,bool cont);
