@@ -55,8 +55,8 @@ ResDecl	res_default_cls         ("cmd:cls-default",  "-F", FileSetOutput::Valida
 /* note: this sorts (add a nosort if necessary) */
 void FileSetOutput::print(FileSet &fs, OutputJob *o) const
 {
-   fs.Sort(sort, sort_casefold);
-   if(sort_dirs_first) fs.Sort(FileSet::DIRSFIRST, false);
+   fs.Sort(sort, sort_casefold, sort_reverse);
+   if(sort_dirs_first) fs.Sort(FileSet::DIRSFIRST, false, sort_reverse);
 
    ColumnOutput c;
 
@@ -137,7 +137,7 @@ void FileSetOutput::print(FileSet &fs, OutputJob *o) const
 	 bool recent = six_months_ago <= f->date;
 
 	 /* We assume all time outputs are equal-width. */
-	 static const char *long_time_format[] = {
+	 const char *const long_time_format[] = {
 	    dcgettext (NULL, "%b %e  %Y", LC_TIME),
 	    dcgettext (NULL, "%b %e %H:%M", LC_TIME)
 	 };
