@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.179 2004/11/16 10:14:41 lav Exp $ */
+/* $Id: Http.cc,v 1.194 2005/05/18 05:57:08 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -31,6 +31,7 @@
 #include <stdarg.h>
 #include <time.h>
 #include <fnmatch.h>
+#include <locale.h>
 #include "Http.h"
 #include "ResMgr.h"
 #include "log.h"
@@ -54,14 +55,12 @@ CDECL char *strptime(const char *buf, const char *format, struct tm *tm);
 #define HTTP_DEFAULT_PROXY_PORT	 "3128"
 #define HTTPS_DEFAULT_PORT	 "443"
 
-static time_t http_atotm (const char *time_string);
-
 /* Some status code validation macros: */
 #define H_20X(x)        (((x) >= 200) && ((x) < 300))
 #define H_PARTIAL(x)    ((x) == 206)
 #define H_REDIRECTED(x) (((x) == 301) || ((x) == 302))
 #define H_EMPTY(x)	(((x) == 204) || ((x) == 205))
-#define H_CONTINUE(x)	((x) == 100)
+#define H_CONTINUE(x)	((x) == 100 || (x) == 102)
 #define H_REQUESTED_RANGE_NOT_SATISFIABLE(x) ((x) == 416)
 
 void Http::Init()
@@ -81,7 +80,6 @@ void Http::Init()
    proto_version=0x10;
    sent_eot=false;
    last_method=0;
-   entity_content_type=0;
 
    default_cwd="/";
 
@@ -106,8 +104,8 @@ void Http::Init()
    use_head=true;
 
    user_agent=0;
-   post_data=0;
-   post=false;
+   special_data=0;
+   special=HTTP_NONE;
 }
 
 Http::Http() : super()
@@ -196,8 +194,6 @@ void Http::ResetRequestData()
    chunk_pos=0;
    chunked_trailer=false;
    seen_ranges_bytes=false;
-   xfree(entity_content_type);
-   entity_content_type=0;
 }
 
 void Http::Close()
@@ -237,9 +233,9 @@ void Http::Close()
    array_send=0;
    no_cache_this=false;
    no_ranges=false;
-   post=false;
-   xfree(post_data);
-   post_data=0;
+   special=HTTP_NONE;
+   xfree(special_data);
+   special_data=0;
    super::Close();
 }
 
@@ -413,9 +409,7 @@ bool Http::ModeSupported()
    {
    case CLOSED:
    case QUOTE_CMD:
-   case RENAME:
    case LIST:
-   case MP_LIST:
    case CHANGE_MODE:
       return false;
    case CONNECT_VERIFY:
@@ -427,11 +421,42 @@ bool Http::ModeSupported()
    case REMOVE_DIR:
    case REMOVE:
    case LONG_LIST:
+   case RENAME:
       return true;
+   case MP_LIST:
+#if USE_EXPAT
+      return QueryBool("use-propfind",hostname);
+#else
+      // without XML parser it is meaningless to retrieve XML file info.
+      return false;
+#endif
    }
    abort(); // should not happen
 }
 
+void Http::DirFile(char *path_base,const char *ecwd,const char *efile)
+{
+   if(efile[0]=='/')
+      strcpy(path_base,efile);
+   else if(efile[0]=='~')
+      sprintf(path_base,"/%s",efile);
+   else if(cwd[0]==0 || ((cwd[0]=='/' || (!hftp && cwd[0]=='~')) && cwd[1]==0))
+      sprintf(path_base,"/%s",efile);
+   else if(cwd[0]=='~')
+      sprintf(path_base,"/%s/%s",ecwd,efile);
+   else
+      sprintf(path_base,"%s/%s",ecwd,efile);
+
+   if(path_base[1]=='~' && path_base[2]=='/')
+      memmove(path_base,path_base+2,strlen(path_base+2)+1);
+   else if(hftp && path_base[1]!='~')
+   {
+      // root directory in ftp urls needs special encoding. (/%2Fpath)
+      memmove(path_base+4,path_base+1,strlen(path_base+1)+1);
+      memcpy(path_base+1,"%2F",3);
+   }
+}
+
 void Http::SendRequest(const char *connection,const char *f)
 {
    char *efile=string_alloca(strlen(f)*3+1);
@@ -442,7 +467,13 @@ void Http::SendRequest(const char *connection,const char *f)
 
    char *pfile=string_alloca(4+3+xstrlen(user)*6+3+xstrlen(pass)*3+1+
 			      strlen(hostname)*3+1+xstrlen(portname)*3+1+
-			      strlen(cwd)*3+1+strlen(efile)+1+6+1);
+			      strlen(ecwd)+1+strlen(efile)+1+6+1);
+
+   const char *allprop=	// PROPFIND request
+      "<?xml version=\"1.0\" ?>"
+      "<propfind xmlns=\"DAV:\">"
+        "<allprop/>"
+      "</propfind>\r\n";
 
    if(proxy && !https)
    {
@@ -473,27 +504,7 @@ void Http::SendRequest(const char *connection,const char *f)
    }
 
    char *path_base=pfile+strlen(pfile);
-
-   if(efile[0]=='/')
-      strcpy(path_base,efile);
-   else if(efile[0]=='~')
-      sprintf(path_base,"/%s",efile);
-   else if(cwd[0]==0 || ((cwd[0]=='/' || (!hftp && cwd[0]=='~')) && cwd[1]==0))
-      sprintf(path_base,"/%s",efile);
-   else if(cwd[0]=='~')
-      sprintf(path_base,"/%s/%s",ecwd,efile);
-   else
-      sprintf(path_base,"%s/%s",ecwd,efile);
-
-   if(path_base[1]=='~' && path_base[2]=='/')
-      memmove(path_base,path_base+2,strlen(path_base+2)+1);
-   else if(hftp && path_base[1]!='~')
-   {
-      // root directory in ftp urls needs special encoding. (/%2Fpath)
-      memmove(path_base+4,path_base+1,strlen(path_base+1)+1);
-      memcpy(path_base+1,"%2F",3);
-   }
-
+   DirFile(path_base,ecwd,efile);
    efile=pfile;
    efile_len=strlen(efile);
 
@@ -502,19 +513,34 @@ void Http::SendRequest(const char *connection,const char *f)
    if(mode==STORE)    // can't seek before writing
       real_pos=pos;
 
+#ifdef DEBUG_MP_LIST
+   if(mode==RETRIEVE && file[0]==0)
+      mode=MP_LIST;
+#endif
+
    switch((open_mode)mode)
    {
    case CLOSED:
    case CONNECT_VERIFY:
+      abort(); // cannot happen
+
    case QUOTE_CMD:
-      if(post)
+      switch(special)
       {
-	 entity_size=xstrlen(post_data);
+      case HTTP_POST:
+	 entity_size=xstrlen(special_data);
 	 goto send_post;
+      case HTTP_MOVE:
+      case HTTP_COPY:
+	 SendMethod(special==HTTP_MOVE?"MOVE":"COPY",efile);
+	 Send("Destination: %s\r\n",special_data);
+	 break;
+      case HTTP_NONE:
+	 abort(); // cannot happen
       }
-   case RENAME:
+      break;
+
    case LIST:
-   case MP_LIST:
    case CHANGE_MODE:
       abort(); // unsupported
 
@@ -557,6 +583,7 @@ void Http::SendRequest(const char *connection,const char *f)
 
    case CHANGE_DIR:
    case LONG_LIST:
+   case MP_LIST:
    case MAKE_DIR:
       if(efile_len<1 || efile[efile_len-1]!='/')
 	 strcat(efile,"/");
@@ -566,20 +593,41 @@ void Http::SendRequest(const char *connection,const char *f)
 	 goto retrieve;
       else if(mode==MAKE_DIR)
       {
-	 SendMethod("PUT",efile);   // hope it would work
-	 Send("Content-length: 0\r\n");
+	 if(QueryBool("use-mkcol"))
+	    SendMethod("MKCOL",efile);
+	 else
+	 {
+	    SendMethod("PUT",efile);
+	    Send("Content-Length: 0\r\n");
+	 }
 	 pos=entity_size=0;
       }
+      else if(mode==MP_LIST)
+      {
+	 SendMethod("PROPFIND",efile);
+	 Send("Depth: 1\r\n"); // directory listing required
+	 Send("Content-Type: text/xml\r\n");
+	 Send("Content-Length: %d\r\n",strlen(allprop));
+	 pos=0;
+      }
       break;
 
    case(REMOVE):
    case(REMOVE_DIR):
       SendMethod("DELETE",efile);
+      if(mode==REMOVE)
+	 Send("Depth: 0\r\n"); // deny directory removal
       break;
 
    case ARRAY_INFO:
       SendMethod("HEAD",efile);
       break;
+
+   case RENAME:
+      {
+	 SendMethod("MOVE",efile);
+	 Send("Destination: %s\r\n",GetFileURL(file1));
+      }
    }
    SendAuth();
    if(no_cache || no_cache_this)
@@ -592,12 +640,16 @@ void Http::SendRequest(const char *connection,const char *f)
    if(mode!=ARRAY_INFO || connection)
       Send("Connection: %s\r\n",connection?connection:"close");
    Send("\r\n");
-   if(post)
+   if(special==HTTP_POST)
    {
-      if(post_data)
-	 Send("%s",post_data);
+      if(special_data)
+	 Send("%s",special_data);
       entity_size=NO_SIZE;
    }
+   else if(mode==MP_LIST)
+   {
+      Send("%s",allprop);
+   }
 
    keep_alive=false;
    chunked=false;
@@ -626,6 +678,31 @@ void Http::SendArrayInfoRequest()
    }
 }
 
+static const char *extract_quoted_header_value(const char *value)
+{
+   static char *value1;
+   xfree(value1);
+   if(*value=='"')
+   {
+      value++;
+      value1=(char*)xmalloc(strlen(value));
+      char *store=value1;
+      while(*value && *value!='"')
+      {
+	 if(*value=='\\' && value[1])
+	    value++;
+	 *store++=*value++;
+      }
+   }
+   else
+   {
+      int end=strcspn(value,"()<>@,;:\\\"/[]?={} \t");
+      value1=xstrdup(value);
+      value1[end]=0;
+   }
+   return value1;
+}
+
 void Http::HandleHeaderLine(const char *name,const char *value)
 {
    if(!strcasecmp(name,"Content-length"))
@@ -673,7 +750,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    }
    if(!strcasecmp(name,"Last-Modified"))
    {
-      time_t t=http_atotm(value);
+      time_t t=Http::atotm(value);
       if(opt_date && H_20X(status_code))
 	 *opt_date=t;
 
@@ -743,26 +820,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       const char *filename=strstr(value,"filename=");
       if(!filename)
 	 return;
-      filename+=9;
-      if(*filename=='"')
-      {
-	 filename++;
-	 char *filename1=string_alloca(strlen(filename));
-	 while(*filename && *filename!='"')
-	 {
-	    if(*filename=='\\' && filename[1])
-	       filename++;
-	    *filename1++=*filename++;
-	 }
-	 filename=filename1;
-      }
-      else
-      {
-	 int end=strcspn(filename,"()<>@,;:\\\"/[]?={} \t");
-	 char *filename1=alloca_strdup(filename);
-	 filename1[end]=0;
-	 filename=filename1;
-      }
+      filename=extract_quoted_header_value(filename+9);
       SetSuggestedFileName(filename);
       return;
    }
@@ -770,7 +828,13 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    {
       xfree(entity_content_type);
       entity_content_type=xstrdup(value);
-//       const char *cs=strstr(value,"charset=");
+      const char *cs=strstr(value,"charset=");
+      if(cs)
+      {
+	 cs=extract_quoted_header_value(cs+8);
+	 xfree(entity_charset);
+	 entity_charset=xstrdup(cs);
+      }
    }
 }
 
@@ -847,7 +911,7 @@ int Http::Do()
 	 Disconnect();
 	 return m;
       }
-      TimeoutS(idle_start+idle-now);
+      TimeoutS(idle_start+idle-time_t(now));
    }
 
    if(home==0)
@@ -867,14 +931,25 @@ int Http::Do()
 	 state=DONE;
 	 return MOVED;
       }
-      if(!hftp && mode==QUOTE_CMD && !post)
+      if(!hftp && mode==QUOTE_CMD && !special)
       {
       handle_quote_cmd:
 	 if(file && !strncasecmp(file,"Set-Cookie ",11))
 	    SetCookie(file+11);
 	 else if(file && !strncasecmp(file,"POST ",5))
+	    special=HTTP_POST;
+	 else if(file && !strncasecmp(file,"COPY ",5))
+	    special=HTTP_COPY;
+	 else if(file && !strncasecmp(file,"MOVE ",5))
+	    special=HTTP_MOVE;
+	 else
 	 {
-	    // POST encoded_path data
+	    SetError(NOT_SUPP,0);
+	    return MOVED;
+	 }
+	 if(special)
+	 {
+	    // METHOD encoded_path data
 	    const char *scan=file+5;
 	    while(*scan==' ')
 	       scan++;
@@ -903,21 +978,15 @@ int Http::Do()
 	    scan=strchr(scan,' ');
 	    while(scan && *scan==' ')
 	       scan++;
-	    post=true;
-	    post_data=xstrdup(scan);
+	    special_data=xstrdup(scan);
 	    xfree(file_url);
 	    file_url=path;
 	    return MOVED;
 	 }
-	 else
-	 {
-	    SetError(NOT_SUPP,0);
-	    return MOVED;
-	 }
 	 state=DONE;
 	 return MOVED;
       }
-      if(!post && !ModeSupported())
+      if(!special && !ModeSupported())
       {
 	 SetError(NOT_SUPP);
 	 return MOVED;
@@ -1022,7 +1091,7 @@ int Http::Do()
 
       m=MOVED;
       state=CONNECTED;
-#ifdef USE_SSL
+#if USE_SSL
       if(proxy?!strncmp(proxy,"https://",8):https)
       {
 	 MakeSSLBuffers();
@@ -1034,7 +1103,7 @@ int Http::Do()
 	    new FDStream(sock,"<output-socket>"),IOBuffer::PUT);
 	 recv_buf=new IOBufferFDStream(
 	    new FDStream(sock,"<input-socket>"),IOBuffer::GET);
-#ifdef USE_SSL
+#if USE_SSL
 	 if(proxy && https)
 	 {
 	    // have to setup a tunnel.
@@ -1055,7 +1124,7 @@ int Http::Do()
       if(mode==CONNECT_VERIFY)
 	 return MOVED;
 
-      if(mode==QUOTE_CMD && !post)
+      if(mode==QUOTE_CMD && !special)
 	 goto handle_quote_cmd;
       if(recv_buf->Eof())
       {
@@ -1065,7 +1134,7 @@ int Http::Do()
       }
       if(mode==CLOSED)
 	 return m;
-      if(!post && !ModeSupported())
+      if(!special && !ModeSupported())
       {
 	 SetError(NOT_SUPP);
 	 return MOVED;
@@ -1096,7 +1165,7 @@ int Http::Do()
    case RECEIVING_HEADER:
       if(send_buf->Error() || recv_buf->Error())
       {
-	 if((mode==STORE || post) && status_code && !H_20X(status_code))
+	 if((mode==STORE || special) && status_code && !H_20X(status_code))
 	    goto pre_RECEIVING_BODY;   // assume error.
       handle_buf_error:
 	 if(send_buf->Error())
@@ -1144,7 +1213,7 @@ int Http::Do()
 	       {
 		  if(H_20X(status_code))
 		  {
-#ifdef USE_SSL
+#if USE_SSL
 		     if(https)
 			MakeSSLBuffers();
 #endif
@@ -1379,8 +1448,23 @@ int Http::Do()
 	 }
 	 else
 	 {
-	    if(file && file[0])
-	       sprintf(err,"%s (%s)",status+status_consumed,file);
+	    const char *closure=file;
+	    if(status_code==400  // Bad request
+	    || status_code==405  // Method Not Allowed
+	    || status_code==501) // Not Implemented
+	    {
+	       if(status_code==400 || status_code==501)
+	       {
+		  if(!xstrcmp(last_method,"PROPFIND"))
+		     ResMgr::Set("http:use-propfind",hostname,"no");
+		  if(!xstrcmp(last_method,"MKCOL"))
+		     ResMgr::Set("http:use-mkcol",hostname,"no");
+	       }
+	       code=NOT_SUPP;
+	       closure=last_method;
+	    }
+	    if(closure && closure[0])
+	       sprintf(err,"%s (%s)",status+status_consumed,closure);
 	    else
 	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd,
 				       (last_char(cwd)=='/')?"":"/");
@@ -1473,7 +1557,7 @@ void  Http::ClassInit()
    // register the class
    Register("http",Http::New);
    Register("hftp",HFtp::New);
-#ifdef USE_SSL
+#if USE_SSL
    Register("https",Https::New);
 #endif
 }
@@ -1649,6 +1733,9 @@ int Http::Done()
       return OK;
    if(mode==CONNECT_VERIFY && (peer || sock!=-1))
       return OK;
+   if((mode==REMOVE || mode==REMOVE_DIR || mode==RENAME)
+   && state==RECEIVING_BODY)
+      return OK;
    return IN_PROGRESS;
 }
 
@@ -2040,7 +2127,7 @@ void Http::SetCookie(const char *value_const)
    xfree(c);
 }
 
-#ifdef USE_SSL
+#if USE_SSL
 #undef super
 #define super Http
 Https::Https()
@@ -2064,10 +2151,9 @@ void Http::MakeSSLBuffers()
    Delete(send_buf);
    Delete(recv_buf);
 
-   SSL *ssl=lftp_ssl_new(sock,hostname);
-   IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT,hostname);
-   IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET,hostname);
-   send_buf_ssl->DoConnect();
+   lftp_ssl *ssl=new lftp_ssl(sock,lftp_ssl::CLIENT,hostname);
+   IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT);
+   IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET);
    recv_buf_ssl->CloseLater();
    send_buf=send_buf_ssl;
    recv_buf=recv_buf_ssl;
@@ -2172,8 +2258,8 @@ check_end (const char *p)
    Marcus Hennecke's atotm(), which is forgiving, fast, to-the-point,
    and does not use strptime().  atotm() is to be found in the sources
    of `phttpd', a little-known HTTP server written by Peter Erikson.  */
-static time_t
-http_atotm (const char *time_string)
+time_t
+Http::atotm (const char *time_string)
 {
   struct tm t;
 
@@ -2209,17 +2295,23 @@ http_atotm (const char *time_string)
      whitespace instead of just one (it works that way on all the
      systems I've tested it on).  */
 
-  /* RFC1123: Thu, 29 Jan 1998 22:12:57 */
-  if (check_end (strptime (time_string, "%a, %d %b %Y %T", &t)))
-    return mktime_from_utc (&t);
-  /* RFC850:  Thu, 29-Jan-98 22:12:57 */
-  if (check_end (strptime (time_string, "%a, %d-%b-%y %T", &t)))
-    return mktime_from_utc (&t);
-  /* asctime: Thu Jan 29 22:12:57 1998 */
-  if (check_end (strptime (time_string, "%a %b %d %T %Y", &t)))
-    return mktime_from_utc (&t);
-  /* Failure.  */
-  return -1;
+   time_t ut=-1;
+
+   setlocale(LC_TIME,"C"); // we need english month and week day names
+
+   /* RFC1123: Thu, 29 Jan 1998 22:12:57 */
+   if (check_end (strptime (time_string, "%a, %d %b %Y %T", &t)))
+      ut=mktime_from_utc (&t);
+   /* RFC850:  Thu, 29-Jan-98 22:12:57 */
+   else if (check_end (strptime (time_string, "%a, %d-%b-%y %T", &t)))
+      ut=mktime_from_utc (&t);
+   /* asctime: Thu Jan 29 22:12:57 1998 */
+   else if (check_end (strptime (time_string, "%a %b %d %T %Y", &t)))
+      ut=mktime_from_utc (&t);
+
+   setlocale(LC_TIME,"");  // restore locale
+
+   return ut;
 }
 
 
