@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.78 2003/12/15 06:56:08 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.86 2005/04/29 05:42:39 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -939,7 +939,7 @@ parse_url_again:
       show_in_list=false;  // makes apache listings look better.
 
    skip_len=tag_len;
-   if(list && show_in_list)
+   // try to find file info
    {
       const char *more1;
       char *str,*str_with_tags, *str2;
@@ -1032,16 +1032,13 @@ parse_url_again:
       if(try_csm_proxy(info,str) && info.validate()) goto got_info;
 
    add_file_no_info:
+      if(!list || !show_in_list)
+	 goto info_done;
       sprintf(line_add,"%s  --  %s",
 	    info.is_directory?"drwxr-xr-x":"-rw-r--r--",link_target);
       goto append_type_maybe;
 
    got_info:
-      if(info_string)
-      {
-	 sprintf(line_add,"%.*s %s",info_string_len,info_string,link_target);
-	 goto append_symlink_maybe;
-      }
       if(info.month==-1)
 	 info.month=parse_month(info.month_name);
       if(info.month>=0)
@@ -1050,6 +1047,38 @@ parse_url_again:
 	 if(info.year==-1)
 	    info.year=guess_year(info.month,info.day,info.hour,info.minute);
       }
+      if(info.year!=-1 && info.month!=-1 && info.day!=-1)
+      {
+	 struct tm tm;
+	 memset(&tm,0,sizeof(tm));
+	 tm.tm_year=info.year-1900;
+	 tm.tm_mon=info.month-1;
+	 tm.tm_mday=info.day;
+	 tm.tm_hour=12;
+	 info.date_prec=43200;
+	 if(info.hour!=-1 && info.minute!=-1)
+	 {
+	    tm.tm_hour=info.hour;
+	    tm.tm_min=info.minute;
+	    tm.tm_sec=30;
+	    info.date_prec=30;
+	    if(info.second!=-1)
+	    {
+	       tm.tm_sec=info.second;
+	       info.date_prec=0;
+	    }
+	 }
+	 info.date=mktime_from_utc(&tm);
+      }
+      if(info.size==-1)
+      {
+	 if(strspn(info.size_str,"0123456789")==strlen(info.size_str))
+	 {
+	    long long size_ll=0;
+	    sscanf(info.size_str,"%lld",&size_ll);
+	    info.size=size_ll;
+	 }
+      }
       if(info.perms[0]==0)
       {
 	 if(info.is_directory)
@@ -1059,6 +1088,14 @@ parse_url_again:
 	 else
 	    strcpy(info.perms,"-rw-r--r--");
       }
+      if(!list || !show_in_list)
+	 goto info_done;
+
+      if(info_string)
+      {
+	 sprintf(line_add,"%.*s %s",info_string_len,info_string,link_target);
+	 goto append_symlink_maybe;
+      }
 
       sprintf(line_add,"%s  %11s  %04d-%s-%02d",
 	 info.perms,info.size_str,info.year,info.month_name,info.day);
@@ -1088,12 +1125,10 @@ parse_url_again:
 
       type = FileInfo::NORMAL;
 
-      if (color) {
-	  if(info.is_directory)
-	      type = FileInfo::DIRECTORY;
-	  else if(info.is_sym_link && !info.sym_link)
-	      type = FileInfo::SYMLINK;
-      }
+       if(info.is_directory)
+	   type = FileInfo::DIRECTORY;
+       else if(info.is_sym_link && !info.sym_link)
+	   type = FileInfo::SYMLINK;
 
       if (color && FileInfo::NORMAL != type && all_links && !all_links->FindByName(link_target)) {
 	  list->Put(line_add);
@@ -1120,12 +1155,11 @@ parse_url_again:
       if(!all_links->FindByName(link_target))
       {
 	 list->Put(line_add);
-	 FileInfo *fi=new FileInfo;
-	 fi->SetName(link_target);
+	 FileInfo *fi=new FileInfo(link_target);
 	 all_links->Add(fi);
       }
    }
-
+info_done:
    if(set && link_target[0]!='/' && link_target[0]!='~')
    {
       char *slash=strchr(link_target,'/');
@@ -1135,8 +1169,7 @@ parse_url_again:
 	 info.is_directory=true;
       }
 
-      FileInfo *fi=new FileInfo;
-      fi->SetName(link_target);
+      FileInfo *fi=new FileInfo(link_target);
       if(info.sym_link)
 	 fi->SetSymlink(info.sym_link);
       else
@@ -1151,7 +1184,7 @@ parse_url_again:
 	 fi->SetSize(info.size);
       if(info.perms[0])
       {
-	 int m=parse_perms(info.perms);
+	 int m=parse_perms(info.perms+1);
 	 if(m>=0)
 	    fi->SetMode(m);
       }
@@ -1195,12 +1228,33 @@ int HttpDirList::Do()
 	 buf->Put(curr);
 	 buf->Put(":\n");
       }
+      if(curr_url)
+	 delete curr_url;
+      curr_url=new ParsedURL(session->GetFileURL(curr));
+      if(mode==FA::RETRIEVE)
+      {
+	 // strip file name, directory remains.
+	 char *slash=strrchr(curr_url->path,'/');
+	 if(slash && slash>curr_url->path)
+	    *slash=0;
+      }
 
+   retry:
       const char *cache_buffer=0;
       int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,curr,mode,
+      int err;
+      if(use_cache && LsCache::Find(session,curr,mode,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
+	 if(err)
+	 {
+	    if(mode==FA::MP_LIST)
+	    {
+	       mode=FA::LONG_LIST;
+	       goto retry;
+	    }
+	    SetErrorCached(cache_buffer);
+	 }
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
@@ -1213,16 +1267,6 @@ int HttpDirList::Do()
 	 if(LsCache::IsEnabled())
 	    ubuf->Save(LsCache::SizeLimit());
       }
-      if(curr_url)
-	 delete curr_url;
-      curr_url=new ParsedURL(session->GetFileURL(curr));
-      if(mode==FA::RETRIEVE)
-      {
-	 // strip file name, directory remains.
-	 char *slash=strrchr(curr_url->path,'/');
-	 if(slash && slash>curr_url->path)
-	    *slash=0;
-      }
    }
 
    const char *b;
@@ -1230,8 +1274,7 @@ int HttpDirList::Do()
    ubuf->Get(&b,&len);
    if(b==0) // eof
    {
-      LsCache::Add(session,curr,mode,ubuf);
-
+      LsCache::Add(session,curr,mode,FA::OK,ubuf);
       Delete(ubuf);
       ubuf=0;
       return MOVED;
@@ -1239,15 +1282,34 @@ int HttpDirList::Do()
 
    int m=STALL;
 
-   int n=parse_html(b,len,ubuf->Eof(),buf,0,&all_links,curr_url,&base_href,&ls_options, color);
-   if(n>0)
+reparse:
+   if(mode!=FA::MP_LIST || parse_as_html)
    {
-      ubuf->Skip(n);
-      m=MOVED;
+      int n=parse_html(b,len,ubuf->Eof(),buf,0,&all_links,curr_url,&base_href,&ls_options, color);
+      if(n>0)
+      {
+	 ubuf->Skip(n);
+	 m=MOVED;
+      }
+   }
+   else
+   {
+      ParsePropsFormat(b,len,ubuf->Eof());
+      if(parse_as_html)
+	 goto reparse;
+      ubuf->Skip(len);
    }
 
    if(ubuf->Error())
    {
+      LsCache::Add(session,curr,mode,session->GetErrorCode(),ubuf);
+      if(mode==FA::MP_LIST)
+      {
+	 mode=FA::LONG_LIST;
+	 Delete(ubuf);
+	 ubuf=0;
+	 goto retry;
+      }
       SetError(ubuf->ErrorText());
       m=MOVED;
    }
@@ -1259,7 +1321,12 @@ HttpDirList::HttpDirList(ArgV *a,FileAccess *fa)
 {
    session=fa;
    ubuf=0;
-   mode=FA::LONG_LIST;
+   mode=FA::MP_LIST;
+   parse_as_html=false;
+#if USE_EXPAT
+   xml_p=0;
+   xml_ctx=0;
+#endif
    args->rewind();
    int opt;
    while((opt=args->getopt("faCFl"))!=EOF)
@@ -1292,6 +1359,7 @@ HttpDirList::HttpDirList(ArgV *a,FileAccess *fa)
 
 HttpDirList::~HttpDirList()
 {
+   ParsePropsFormat(0,0,true);
    Delete(ubuf);
    if(curr_url)
       delete curr_url;
@@ -1327,6 +1395,13 @@ void HttpDirList::Resume()
 // HttpListInfo implementation
 FileSet *HttpListInfo::Parse(const char *b,int len)
 {
+   if(mode==FA::MP_LIST)
+   {
+      FileSet *fs=ParseProps(b,len,session->GetCwd());
+      if(!fs)
+	 mode=FA::LONG_LIST;
+      return fs;
+   }
    return session->ParseLongList(b,len);
 }
 
