@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.116 2004/12/17 06:51:47 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.121 2005/05/13 13:33:00 lav Exp $ */
 
 #include <config.h>
 
@@ -39,6 +39,10 @@
 #include "url.h"
 #include "CopyJob.h"
 #include "pgetJob.h"
+#include "log.h"
+
+#define set_state(s) do { state=(s); \
+   Log::global->Format(11,"mirror(%p) enters state %s\n", this, #s); } while(0)
 
 void  MirrorJob::PrintStatus(int v,const char *tab)
 {
@@ -305,7 +309,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    cp->SetParentFg(this);
 	    cp->cmdline=xasprintf("\\transfer %s",file->name);
 	 }
-	 state=WAITING_FOR_TRANSFER;
+	 set_state(WAITING_FOR_TRANSFER);
 	 break;
       }
       case(FileInfo::DIRECTORY):
@@ -383,6 +387,9 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 if(flags&NO_SYMLINKS)
 	    goto skip;
 
+	 if(!file->symlink)
+	    goto skip;
+
 	 if(!target_is_local)
 	 {
 	    // can't create symlink remotely (FIXME?)
@@ -529,7 +536,7 @@ void MirrorJob::HandleChdir(FileAccess * &session, int &redirections)
       eprintf("mirror: %s\n",session->StrError(res));
       stats.error_count++;
       transfer_count-=root_transfer_count;
-      state=FINISHING;
+      set_state(FINISHING);
       source_session->Close();
       target_session->Close();
       return;
@@ -554,7 +561,7 @@ void MirrorJob::HandleListInfoCreation(FileAccess * &session,ListInfo * &list_in
       eprintf(_("mirror: protocol `%s' is not suitable for mirror\n"),
 	       session->GetProto());
       transfer_count-=root_transfer_count;
-      state=FINISHING;
+      set_state(FINISHING);
       return;
    }
    list_info->UseCache(use_cache);
@@ -577,7 +584,7 @@ void MirrorJob::HandleListInfo(ListInfo * &list_info, FileSet * &set)
       eprintf("mirror: %s\n",list_info->ErrorText());
       stats.error_count++;
       transfer_count-=root_transfer_count;
-      state=FINISHING;
+      set_state(FINISHING);
       Delete(source_list_info);
       source_list_info=0;
       Delete(target_list_info);
@@ -603,7 +610,7 @@ int   MirrorJob::Do()
       source_session->Chdir(source_dir);
       source_redirections=0;
       Roll(source_session);
-      state=CHANGING_DIR_SOURCE;
+      set_state(CHANGING_DIR_SOURCE);
       m=MOVED;
       /*fallthrough*/
    case(CHANGING_DIR_SOURCE):
@@ -663,7 +670,7 @@ int   MirrorJob::Do()
 	    goto pre_CHANGING_DIR_TARGET;
       }
       target_session->Mkdir(target_dir,(parent_mirror==0));
-      state=MAKE_TARGET_DIR;
+      set_state(MAKE_TARGET_DIR);
       m=MOVED;
       /*fallthrough*/
    case(MAKE_TARGET_DIR):
@@ -676,7 +683,7 @@ int   MirrorJob::Do()
       target_session->Chdir(target_dir);
       target_redirections=0;
       Roll(target_session);
-      state=CHANGING_DIR_TARGET;
+      set_state(CHANGING_DIR_TARGET);
       m=MOVED;
       /*fallthrough*/
    case(CHANGING_DIR_TARGET):
@@ -689,7 +696,8 @@ int   MirrorJob::Do()
       xfree(target_dir);
       target_dir=xstrdup(target_session->GetCwd());
 
-      state=GETTING_LIST_INFO;
+   pre_GETTING_LIST_INFO:
+      set_state(GETTING_LIST_INFO);
       m=MOVED;
       HandleListInfoCreation(source_session,source_list_info,source_relative_dir);
       HandleListInfoCreation(target_session,target_list_info,target_relative_dir);
@@ -697,9 +705,8 @@ int   MirrorJob::Do()
       {
 	 Delete(source_list_info); source_list_info=0;
 	 Delete(target_list_info); target_list_info=0;
-	 return MOVED;
       }
-      /*fallthrough*/
+      return m;	  // give time to other tasks
    case(GETTING_LIST_INFO):
       HandleListInfo(source_list_info,source_set);
       HandleListInfo(target_list_info,target_set);
@@ -717,16 +724,16 @@ int   MirrorJob::Do()
       to_rm_mismatched->Count(&stats.del_dirs,&stats.del_files,&stats.del_symlinks,&stats.del_files);
       to_rm_mismatched->rewind();
       transfer_count-=root_transfer_count; // leave room for transfers.
-      state=TARGET_REMOVE_OLD_FIRST;
-      return MOVED;
+      set_state(TARGET_REMOVE_OLD_FIRST);
+      goto TARGET_REMOVE_OLD_FIRST_label;
 
    pre_WAITING_FOR_TRANSFER:
       to_transfer->rewind();
-      state=WAITING_FOR_TRANSFER;
+      set_state(WAITING_FOR_TRANSFER);
       m=MOVED;
       /*fallthrough*/
    case(WAITING_FOR_TRANSFER):
-      while(j=FindDoneAwaitedJob())
+      while((j=FindDoneAwaitedJob())!=0)
       {
 	 if(j->ExitCode()!=0)
 	    stats.error_count++;
@@ -753,12 +760,13 @@ int   MirrorJob::Do()
    pre_TARGET_REMOVE_OLD:
       if(flags&REMOVE_FIRST)
 	 goto pre_TARGET_CHMOD;
-      state=TARGET_REMOVE_OLD;
+      set_state(TARGET_REMOVE_OLD);
       m=MOVED;
       /*fallthrough*/
    case(TARGET_REMOVE_OLD):
    case(TARGET_REMOVE_OLD_FIRST):
-      while(j=FindDoneAwaitedJob())
+   TARGET_REMOVE_OLD_FIRST_label:
+      while((j=FindDoneAwaitedJob())!=0)
       {
 	 RemoveWaiting(j);
 	 Delete(j);
@@ -840,11 +848,11 @@ int   MirrorJob::Do()
 	 goto pre_FINISHING;
 
       to_transfer->rewind();
-      state=TARGET_CHMOD;
+      set_state(TARGET_CHMOD);
       m=MOVED;
       /*fallthrough*/
    case(TARGET_CHMOD):
-      while(j=FindDoneAwaitedJob())
+      while((j=FindDoneAwaitedJob())!=0)
       {
 	 RemoveWaiting(j);
 	 Delete(j);
@@ -914,11 +922,11 @@ int   MirrorJob::Do()
 	 if(flags&ALLOW_CHOWN)
 	    same->LocalChown(target_dir);
       }
-      state=FINISHING;
+      set_state(FINISHING);
       m=MOVED;
       /*fallthrough*/
    case(FINISHING):
-      while(j=FindDoneAwaitedJob())
+      while((j=FindDoneAwaitedJob())!=0)
       {
 	 RemoveWaiting(j);
 	 Delete(j);
@@ -931,7 +939,18 @@ int   MirrorJob::Do()
       transfer_count++; // parent mirror will decrement it.
       if(parent_mirror)
 	 parent_mirror->stats.Add(stats);
-      state=DONE;
+      else
+      {
+	 if((flags&LOOP) && stats.HaveSomethingDone() && !stats.error_count)
+	 {
+	    set_state(DONE);
+	    PrintStatus(0,"");
+	    printf(_("Retrying mirror...\n"));
+	    stats.Reset();
+	    goto pre_GETTING_LIST_INFO;
+	 }
+      }
+      set_state(DONE);
       m=MOVED;
       /*fallthrough*/
    case(DONE):
@@ -976,7 +995,7 @@ MirrorJob::MirrorJob(MirrorJob *parent,
 
    exclude=0;
 
-   state=INITIAL_STATE;
+   set_state(INITIAL_STATE);
 
    newer_than=(time_t)-1;
 
@@ -1115,11 +1134,17 @@ void MirrorJob::Bg()
 }
 
 MirrorJob::Statistics::Statistics()
+{
+   Reset();
+   error_count=0;
+   bytes=0;
+   time=0;
+}
+void MirrorJob::Statistics::Reset()
 {
    tot_files=new_files=mod_files=del_files=
    tot_symlinks=new_symlinks=mod_symlinks=del_symlinks=
-   dirs=del_dirs=
-   error_count=0;
+   dirs=del_dirs=0;
 }
 void MirrorJob::Statistics::Add(const Statistics &s)
 {
@@ -1134,6 +1159,12 @@ void MirrorJob::Statistics::Add(const Statistics &s)
    dirs        +=s.dirs;
    del_dirs    +=s.del_dirs;
    error_count +=s.error_count;
+   bytes       +=s.bytes;
+   time	       +=s.time;
+}
+bool MirrorJob::Statistics::HaveSomethingDone()
+{
+   return new_files|mod_files|del_files|new_symlinks|mod_symlinks|del_symlinks|del_dirs;
 }
 
 char *MirrorJob::SetScriptFile(const char *n)
@@ -1189,6 +1220,7 @@ CMD(mirror)
       {"delete-first",no_argument,0,256+'e'},
       {"use-pget-n",optional_argument,0,256+'p'},
       {"no-symlinks",no_argument,0,256+'y'},
+      {"loop",no_argument,0,256+'l'},
       {0}
    };
 
@@ -1212,6 +1244,9 @@ CMD(mirror)
    PatternSet *exclude=0;
    const char *default_exclude=ResMgr::Query("mirror:exclude-regex",0);
 
+   if(!ResMgr::QueryBool("mirror:set-permissions",0))
+      flags|=MirrorJob::NO_PERMS;
+
    args->rewind();
    while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LPa",mirror_opts,0))!=EOF)
    {
@@ -1342,6 +1377,9 @@ CMD(mirror)
       case(256+'y'):
 	 flags|=MirrorJob::NO_SYMLINKS;
 	 break;
+      case(256+'l'):
+	 flags|=MirrorJob::LOOP;
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
       no_job:
