@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.74 2004/12/01 12:17:43 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.78 2005/04/07 15:30:22 lav Exp $ */
 
 #include <config.h>
 
@@ -27,6 +27,12 @@
 #include <math.h>
 #include <sys/types.h>
 #include <netinet/in.h>
+#ifdef HAVE_NETINET_IN_SYSTM_H
+# include <netinet/in_systm.h>
+#endif
+#ifdef HAVE_NETINET_IP_H
+# include <netinet/ip.h>
+#endif
 #ifdef HAVE_NETINET_TCP_H
 # include <netinet/tcp.h>
 #endif
@@ -137,6 +143,22 @@ void NetAccess::KeepAlive(int sock)
 {
    setsockopt(sock,SOL_SOCKET,SO_KEEPALIVE,(char*)&one,sizeof(one));
 }
+void NetAccess::MinimizeLatency(int sock)
+{
+   int tos;
+#ifdef IP_TOS
+   tos = IPTOS_LOWDELAY;
+   setsockopt(sock, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int));
+#endif
+}
+void NetAccess::MaximizeThroughput(int sock)
+{
+   int tos;
+#ifdef IP_TOS
+   tos = IPTOS_THROUGHPUT;
+   setsockopt(sock, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int));
+#endif
+}
 void NetAccess::ReuseAddress(int sock)
 {
    setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,(char*)&one,sizeof(one));
@@ -692,9 +714,20 @@ do_again:
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,"",mode,
+      int err;
+      if(use_cache && LsCache::Find(session,"",mode,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
+	 if(err)
+	 {
+	    if(mode==FA::MP_LIST)
+	    {
+	       mode=FA::LONG_LIST;
+	       goto do_again;
+	    }
+	    SetErrorCached(cache_buffer);
+	    return MOVED;
+	 }
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
@@ -716,16 +749,14 @@ do_again:
    {
       if(ubuf->Error())
       {
-	 if(session->GetErrorCode()==FA::NOT_SUPP)
+	 LsCache::Add(session,"",mode,session->GetErrorCode(),ubuf);
+	 if(mode==FA::MP_LIST)
 	 {
-	    if(mode==FA::MP_LIST)
-	    {
-	       mode=FA::LONG_LIST;
-	       delete ubuf;
-	       ubuf=0;
-	       m=MOVED;
-	       goto do_again;
-	    }
+	    mode=FA::LONG_LIST;
+	    delete ubuf;
+	    ubuf=0;
+	    m=MOVED;
+	    goto do_again;
 	 }
 	 SetError(ubuf->ErrorText());
 	 Delete(ubuf);
@@ -736,7 +767,7 @@ do_again:
       if(!ubuf->Eof())
 	 return m;
 
-      LsCache::Add(session,"",mode,ubuf);
+      LsCache::Add(session,"",mode,FA::OK,ubuf);
 
       // now we have all the index in ubuf; parse it.
       const char *b;
