@@ -45,7 +45,15 @@ public:
    void Init();
    PollVec();
 
-   void	 Empty();
+   void	 Empty()
+      {
+	 fds_num=0;
+	 timeout=-1;
+      }
+   bool IsEmpty()
+      {
+	 return fds_num==0 && timeout==-1;
+      }
    void	 Merge(const PollVec&);
    void	 Block() const;
 
