@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.cc,v 1.35 2004/11/26 10:14:56 lav Exp $ */
+/* $Id: SFtp.cc,v 1.45 2005/05/20 10:14:32 lav Exp $ */
 
 #include <config.h>
 #include "SFtp.h"
@@ -92,7 +92,7 @@ int SFtp::Do()
 	 Disconnect();
 	 return m;
       }
-      TimeoutS(idle_start+idle-now);
+      TimeoutS(idle_start+idle-time_t(now));
    }
 
    if(Error())
@@ -121,7 +121,8 @@ int SFtp::Do()
       BumpEventTime(pty_recv_buf->EventTime());
 
    // check for timeout only if there should be connection activity.
-   if(state!=DISCONNECTED && mode!=CLOSED && CheckTimeout())
+   if(state!=DISCONNECTED && state!=CONNECTED
+   && mode!=CLOSED && CheckTimeout())
       return MOVED;
 
    if((state==FILE_RECV || state==FILE_SEND)
@@ -162,7 +163,7 @@ int SFtp::Do()
       const char *init=Query("server-program",hostname);
       const char *prog=Query("connect-program",hostname);
       if(!prog || !prog[0])
-	 prog="ssh -ax";
+	 prog="ssh -a -x";
       char *a=alloca_strdup(prog);
       ArgV *cmd=new ArgV;
       for(a=strtok(a," "); a; a=strtok(0," "))
@@ -713,6 +714,9 @@ void SFtp::Close()
    // don't need these out-of-order packets anymore
    while(ooo_chain)
       DeleteExpect(&ooo_chain);
+   if(recv_buf && !suspended)
+      recv_buf->Resume();
+   recv_buf_suspended=false;
 }
 
 int SFtp::HandlePty()
@@ -893,6 +897,15 @@ void SFtp::HandleExpect(Expect *e)
 	    d->GetData(&b,&s);
 	    Log::global->Format(9,"---- data packet: pos=%lld, size=%d\n",(long long)r->pos,s);
 	    file_buf->Put(b,s);
+	    if(r->len > unsigned(s))   // received less than requested?
+	    {
+	       // if we have not yet requested next chunk of data,
+	       // then adjust request position, else re-request missed data.
+	       if(r->pos+r->len==request_pos)
+		  request_pos=r->pos+s;
+	       else
+		  SendRequest(new Request_READ(handle,handle_len,r->pos+s,r->len-s),Expect::DATA);
+	    }
 	 }
 	 else
 	 {
@@ -981,8 +994,6 @@ void SFtp::HandleExpect(Expect *e)
 	 if(opt_date)
 	    *opt_date=entity_date;
       }
-      else
-	 SetError(NO_FILE,reply);
       break;
    case Expect::WRITE_STATUS:
       if(reply->TypeIs(SSH_FXP_STATUS))
@@ -1242,7 +1253,12 @@ int SFtp::Buffered()
 {
    if(file_buf==0)
       return 0;
-   return file_buf->Size()+send_buf->Size()*size_write/(size_write+20);
+   off_t b=file_buf->Size()+send_buf->Size()*size_write/(size_write+20);
+   if(b<0)
+      b=0;
+   else if(b>real_pos)
+      b=real_pos;
+   return b;
 }
 int SFtp::StoreStatus()
 {
@@ -1809,6 +1825,10 @@ FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
    const char *name=utf8_to_lc(na->name);
    if(!name || !name[0])
       return 0;
+   if(strchr(name,'/'))
+      return 0;
+   if(name[0]=='~')
+      name=dir_file(".",name);
    FileInfo *fi=new FileInfo(name);
    switch(a->type)
    {
@@ -1878,9 +1898,15 @@ int SFtpDirList::Do()
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,dir,FA::LONG_LIST,
+      int err;
+      if(use_cache && LsCache::Find(session,dir,FA::LONG_LIST,&err,
 				    &cache_buffer,&cache_buffer_size,&fset))
       {
+	 if(err)
+	 {
+	    SetErrorCached(cache_buffer);
+	    return MOVED;
+	 }
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
@@ -1902,7 +1928,7 @@ int SFtpDirList::Do()
    {
       if(!fset && session->IsOpen())
 	 fset=((SFtp*)session)->GetFileSet();
-      LsCache::Add(session,dir,FA::LONG_LIST, ubuf, fset);
+      LsCache::Add(session,dir,FA::LONG_LIST,FA::OK,ubuf,fset);
       if(use_file_set)
       {
 	 fset->Sort(fset->BYNAME,false);
@@ -2011,13 +2037,21 @@ void SFtpDirList::Resume()
 int SFtpListInfo::Do()
 {
    int m=STALL;
+   if(done)
+      return m;
    if(!ubuf)
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,"",FA::LONG_LIST,
+      int err;
+      if(use_cache && LsCache::Find(session,"",FA::LONG_LIST,&err,
 				    &cache_buffer,&cache_buffer_size,&result))
       {
+	 if(err)
+	 {
+	    SetErrorCached(cache_buffer);
+	    return MOVED;
+	 }
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
@@ -2038,7 +2072,7 @@ int SFtpListInfo::Do()
    {
       if(!result && session->IsOpen())
 	 result=((SFtp*)session)->GetFileSet();
-      LsCache::Add(session,"",FA::LONG_LIST, ubuf, result);
+      LsCache::Add(session,"",FA::LONG_LIST,FA::OK,ubuf,result);
       result->ExcludeDots();
       result->Exclude(exclude_prefix,exclude);
       done=true;
