@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SMTask.cc,v 1.30 2004/09/06 09:19:57 lav Exp $ */
+/* $Id: SMTask.cc,v 1.35 2005/04/29 10:31:13 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -30,9 +30,13 @@
 #endif
 
 #include "SMTask.h"
+#include "misc.h"
 
 SMTask	 *SMTask::chain=0;
 SMTask	 *SMTask::current=0;
+SMTask	 **SMTask::stack=0;
+int      SMTask::stack_ptr=0;
+int      SMTask::stack_size=0;
 PollVec	 SMTask::sched_total;
 TimeDate SMTask::now;
 
@@ -56,7 +60,6 @@ SMTask::SMTask()
    suspended=false;
    running=0;
    deleting=false;
-   prev_current=0;
    task_count++;
 #ifdef TASK_DEBUG
    printf("new SMTask %p (count=%d)\n",this,task_count);
@@ -97,10 +100,25 @@ void SMTask::Delete(SMTask *task)
       delete task;
 }
 
+void SMTask::Enter(SMTask *task)
+{
+   if(stack_size<=stack_ptr)
+      stack=(SMTask**)xrealloc(stack,(stack_size+=16)*sizeof(*stack));
+   stack[stack_ptr++]=current;
+   current=task;
+   current->running++;
+}
+void SMTask::Leave(SMTask *task)
+{
+   assert(current==task);
+   current->running--;
+   current=stack[--stack_ptr];
+}
+
 int SMTask::Roll(SMTask *task)
 {
    int m=STALL;
-   if(task->running)
+   if(task->running || task->deleting)
       return m;
    Enter(task);
    while(!task->deleting && task->Do()==MOVED)
@@ -146,8 +164,10 @@ void SMTask::Schedule()
       int res=STALL;
 
       Enter(scan);	// mark it current and running.
+      SMTask *entered=current;
       if(!current->deleting)
 	 res=current->Do(); // let it run unless it is dying.
+      assert(current==entered);
       scan=scan->next;	// move to a next task.
       SMTask *to_delete=0;
       if(current->deleting)
@@ -260,32 +280,5 @@ bool SMTask::NonFatalError(int err)
 
 bool SMTask::TemporaryNetworkError(int err)
 {
-   switch(err)
-   {
-   case(ETIMEDOUT):
-#ifdef ECONNRESET
-   case(ECONNRESET):
-#endif
-   case(ECONNREFUSED):
-#ifdef EHOSTUNREACH
-   case(EHOSTUNREACH):
-#endif
-#ifdef EHOSTDOWN
-   case(EHOSTDOWN):
-#endif
-#ifdef ENETRESET
-   case(ENETRESET):
-#endif
-#ifdef ENETUNREACH
-   case(ENETUNREACH):
-#endif
-#ifdef ENETDOWN
-   case(ENETDOWN):
-#endif
-#ifdef ECONNABORTED
-   case(ECONNABORTED):
-#endif
-      return true;
-   }
-   return false;
+   return temporary_network_error(err);
 }
