@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.218 2005/04/04 09:59:44 lav Exp $ */
+/* $Id: commands.cc,v 1.223 2005/05/19 12:41:11 lav Exp $ */
 
 #include <config.h>
 
@@ -33,6 +33,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <assert.h>
+#ifdef HAVE_DLFCN_H
+# include <dlfcn.h>
+#endif
 
 #include "CmdExec.h"
 #include "GetJob.h"
@@ -740,10 +743,11 @@ Job *CmdExec::builtin_open()
       {"execute",required_argument,0,'e'},
       {"debug",optional_argument,0,'d'},
       {"no-bookmark",no_argument,0,'B'},
+      {"help",no_argument,0,'h'},
       {0,0,0,0}
    };
 
-   while((c=args->getopt_long("u:p:e:dB",open_options,0))!=EOF)
+   while((c=args->getopt_long("u:p:e:dBh",open_options,0))!=EOF)
    {
       switch(c)
       {
@@ -772,6 +776,12 @@ Job *CmdExec::builtin_open()
       case('B'):
 	 no_bm=true;
 	 break;
+      case('h'):
+	 if(!strcmp(op,"lftp"))
+	 {
+	    PrependCmd("help lftp");
+	    return 0;
+	 }
       case('?'):
 	 if(!strcmp(op,"lftp"))
 	    eprintf(_("Try `%s --help' for more information\n"),op);
@@ -2418,11 +2428,70 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("Lftp | Version %s | Copyright (c) 1996-2005 Alexander V. Lukyanov\n"),VERSION);
+      _("LFTP | Version %s | Copyright (c) 1996-2005 Alexander V. Lukyanov\n"),VERSION);
+   printf("\n");
    printf(
-      _("This is free software with ABSOLUTELY NO WARRANTY. See COPYING for details.\n"));
+      _("LFTP is free software, covered by the GNU General Public License, and you are\n"
+      "welcome to change it and/or distribute copies of it under certain conditions.\n"
+      "There is absolutely no warranty for LFTP.  See COPYING for details.\n"));
+   printf("\n");
    printf(
       _("Send bug reports and questions to <%s>.\n"),"lftp@uniyar.ac.ru");
+
+#if defined(HAVE_DLOPEN) && defined(RTLD_DEFAULT)
+   printf("\n");
+   printf(_("Libraries used: "));
+
+   struct VersionInfo
+   {
+      const char *lib_name;
+      const char *symbol;
+      enum type_t { STRING_PTR, FUNC0 } type;
+      const char *skip_prefix;
+      typedef const char *(*func0)(int);
+      const char *query() const
+	 {
+	    void *sym_ptr=dlsym(RTLD_DEFAULT,symbol);
+	    if(!sym_ptr)
+	       return 0;
+	    const char *str=0;
+	    switch(type)
+	    {
+	    case STRING_PTR:
+	       str=*(const char**)sym_ptr;
+	       break;
+	    case FUNC0:
+	       str=((func0)sym_ptr)(0);
+	       break;
+	    }
+	    if(!str)
+	       return 0;
+	    if(skip_prefix && !strncmp(str,skip_prefix,strlen(skip_prefix)))
+	       str+=strlen(skip_prefix);
+	    return str;
+	 }
+   }
+   static const libs[]=
+   {
+      {"Readline",   "rl_library_version",   VersionInfo::STRING_PTR,0},
+      {"Expat",	     "XML_ExpatVersion",     VersionInfo::FUNC0,     "expat_"},
+      {"OpenSSL",    "SSL_version_str",	     VersionInfo::STRING_PTR,"OpenSSL "},
+      {"GnuTLS",     "gnutls_check_version", VersionInfo::FUNC0,     0},
+      {0}
+   };
+
+   bool need_comma=false;
+   for(const VersionInfo *scan=libs; scan->lib_name; scan++)
+   {
+      const char *v=scan->query();
+      if(!v)
+	 continue;
+      printf("%s%s %s",need_comma?", ":"",scan->lib_name,v);
+      need_comma=true;
+   }
+   printf("\n");
+#endif // HAVE_DLOPEN
+
    exit_code=0;
    return 0;
 }
