@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: complete.cc,v 1.67 2004/06/07 14:22:40 lav Exp $ */
+/* $Id: complete.cc,v 1.70 2004/11/03 08:37:44 lav Exp $ */
 
 #include <config.h>
 
@@ -57,7 +57,7 @@ CDECL_END
 #endif
 
 static char *bash_dequote_filename (char *text, int quote_char);
-static int lftp_char_is_quoted(char *string,int eindex);
+static int lftp_char_is_quoted(const char *string,int eindex);
 
 static int len;    // lenght of the word to complete
 static int cindex; // index in completion array
@@ -255,23 +255,59 @@ int ignore_non_dirs(char **matches)
 
 static const char *find_word(const char *p)
 {
-   while(*p && isspace(*p))
+   while(CmdExec::is_space(*p))
       p++;
    return p;
 }
+static bool copy_word_skip(const char **p_in,char *buf,int n)
+{
+   const char *&p=*p_in;
+   char in_quotes=0;
+   for(;;)
+   {
+      if(!*p)
+	 break;
+      if(in_quotes)
+      {
+	 if(*p==in_quotes)
+	 {
+	    in_quotes=0,p++;
+	    continue;
+	 }
+	 else if(*p=='\\' && CmdExec::quotable(p[1],in_quotes))
+	    p++;
+	 if(buf && n>0)
+	    *buf++=*p,n--;
+	 p++;
+	 continue;
+      }
+      if(CmdExec::is_space(*p))
+	 break;
+      if(*p=='\\' && CmdExec::quotable(p[1],in_quotes))
+	 p++;
+      else if(CmdExec::is_quote(*p))
+      {
+	 in_quotes=*p++;
+	 continue;
+      }
+      if(buf && n>0)
+	 *buf++=*p,n--;
+      p++;
+   }
+   if(n>0 && buf)
+      *buf=0;
+   return n>0;
+}
 // returns false when buffer overflows
 static bool copy_word(char *buf,const char *p,int n)
 {
-   while(n>0 && *p && !isspace(*p))
-   {
-      *buf++=*p++;
-      n--;
-   }
-   if(n>0)
-      *buf=0;
-   return n>0;
+   return copy_word_skip(&p,buf,n);
+}
+static const char *skip_word(const char *p)
+{
+   copy_word_skip(&p,0,0);
+   return p;
 }
-
 
 enum completion_type
 {
@@ -279,23 +315,20 @@ enum completion_type
    STRING_ARRAY, VARIABLE, NO_COMPLETION
 };
 
-// cmd: ptr to command line beging completed
+// cmd: ptr to command line being completed
 // start: location of the word being completed
 static completion_type cmd_completion_type(const char *cmd,int start)
 {
-   const char *w=find_word(cmd);
-
-   if(w-cmd == start) // first word is command
-      return COMMAND;
-
-   // try to guess whether the completion word is remote
-
+   const char *w=0;
    char buf[20];  // no commands longer
    TouchedAlias *used_aliases=0;
 
+   // try to guess whether the completion word is remote
    for(;;)
    {
       w=find_word(cmd);
+      if(w-cmd == start) // first word is command
+	 return COMMAND;
       if(w[0]=='!')
 	 shell_cmd=true;
       if(w[0]=='#')
@@ -309,6 +342,7 @@ static completion_type cmd_completion_type(const char *cmd,int start)
 	 return COMMAND;
       if(w[0]=='(')
       {
+	 start-=(w+1-cmd);
 	 cmd=w+1;
 	 continue;
       }
@@ -322,7 +356,11 @@ static completion_type cmd_completion_type(const char *cmd,int start)
       if(alias && !TouchedAlias::IsTouched(alias,used_aliases))
       {
 	 used_aliases=new TouchedAlias(alias,used_aliases);
-	 cmd=alias;
+	 char *cmd1=string_alloca(strlen(alias)+strlen(w)-strlen(buf)+1);
+	 strcpy(cmd1,alias);
+	 strcat(cmd1,w+strlen(buf));
+	 start=start-strlen(buf)+strlen(alias);
+	 cmd=cmd1;
 	 continue;
       }
       const char *full=CmdExec::GetFullCommandName(buf);
@@ -332,11 +370,11 @@ static completion_type cmd_completion_type(const char *cmd,int start)
       break;
    }
 
-   for(char *p=rl_line_buffer+start; p>rl_line_buffer; )
+   for(const char *p=cmd+start; p>cmd; )
    {
       p--;
       if((*p=='>' || *p=='|')
-      && !lftp_char_is_quoted(rl_line_buffer,p-rl_line_buffer))
+      && !lftp_char_is_quoted(cmd,p-cmd))
 	 return LOCAL;
       if(!isspace((unsigned char)*p))
 	 break;
@@ -383,6 +421,8 @@ static completion_type cmd_completion_type(const char *cmd,int start)
    bool was_o=false;
    bool was_N=false;
    bool was_O=false;
+   bool have_R=false;
+   bool have_N=false;
    bool second=false;
    int second_start=-1;
    for(int i=start; i>4; i--)
@@ -410,9 +450,7 @@ static completion_type cmd_completion_type(const char *cmd,int start)
 	 break;
       }
    }
-   w=find_word(cmd);
-   while(*w && !isspace(*w))
-      w++;  // FIXME: handle quotations.
+   w=skip_word(find_word(cmd));
    if(*w)
    {
       w=find_word(w);
@@ -421,6 +459,25 @@ static completion_type cmd_completion_type(const char *cmd,int start)
 	 second=true;
    }
 
+   if(re_match(cmd," -[a-zA-Z]*R[a-zA-Z]* ",0))
+      have_R=true;
+
+   int arg=0;
+   bool opt_stop=false;
+   for(w=cmd; *w; )
+   {
+      w=find_word(w);
+      if(w-cmd==start || w-cmd+CmdExec::is_quote(*w)==start)
+	 break;
+      if(w[0]!='-' || opt_stop)
+	 arg++;
+      else if(re_match(w,"^-[a-zA-Z]*N ",0))
+	 have_N=true;
+      if(!strncmp(w,"-- ",3))
+	 opt_stop=true;
+      w=skip_word(w);
+   }
+
    if(!strcmp(buf,"get")
    || !strcmp(buf,"pget")
    || !strcmp(buf,"get1"))
@@ -442,9 +499,15 @@ static completion_type cmd_completion_type(const char *cmd,int start)
 	 return REMOTE_DIR;
    if(!strcmp(buf,"mirror"))
    {
-      // FIXME: guess -R and take arg number into account.
-      if(!was_N)
-	 return REMOTE_DIR;
+      if(was_N)
+	 return LOCAL;
+      if(have_N)
+	 arg--;
+      if(have_R ^ arg!=1)
+	 return LOCAL_DIR;
+      if(arg>2)
+	 return NO_COMPLETION;
+      return REMOTE_DIR;
    }
    if(!strcmp(buf,"bookmark"))
    {
@@ -1129,7 +1192,7 @@ static int skip_quoted(const char *s, int i, char q)
    return i;
 }
 
-int lftp_char_is_quoted(char *string,int eindex)
+int lftp_char_is_quoted(const char *string,int eindex)
 {
   int i, pass_next;
 
