@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.123 2004/11/15 09:27:56 lav Exp $ */
+/* $Id: ftpclass.h,v 1.126 2005/05/18 05:57:09 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -28,8 +28,8 @@
 
 #include "NetAccess.h"
 
-#ifdef USE_SSL
-# include <openssl/ssl.h>
+#if USE_SSL
+# include "lftp_ssl.h"
 #endif
 
 class IOBufferTelnet : public IOBufferStacked
@@ -104,9 +104,8 @@ class Ftp : public NetAccess
       off_t last_rest;	// last successful REST position.
       off_t rest_pos;	// the number sent with REST command.
 
-#ifdef USE_SSL
-      SSL *control_ssl;
-      SSL *data_ssl;
+#if USE_SSL
+      lftp_ssl *control_ssl;
       char prot;  // current data protection scheme 'C'lear or 'P'rivate
       bool auth_sent;
       bool auth_supported;
@@ -181,7 +180,7 @@ class Ftp : public NetAccess
 	 LANG,
 	 SITE_UTIME,
 	 QUOTED		// check response for any command submitted by QUOTE_CMD
-#ifdef USE_SSL
+#if USE_SSL
 	 ,AUTH_TLS,PROT,SSCN
 #endif
       };
@@ -269,8 +268,7 @@ class Ftp : public NetAccess
 
    void	 HandleTimeout();
 
-#ifdef USE_SSL
-   void	 BlockOnSSL(SSL*);
+#if USE_SSL
 protected:
    bool	 ftps;	  // ssl and prot='P' by default (port 990)
 private:
@@ -516,6 +514,11 @@ public:
 	    return copy_addr_valid && expect->FirstIs(Expect::TRANSFER);
 	 return state==WAITING_STATE && expect->IsEmpty();
       }
+   void CopyCheckTimeout(Ftp *o)
+      {
+	 BumpEventTime(o->event_time);
+	 CheckTimeout();
+      }
 };
 
 class FtpS : public Ftp
