@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.60 2004/04/16 14:27:36 lav Exp $ */
+/* $Id: lftp.cc,v 1.64 2005/05/18 13:25:17 lav Exp $ */
 
 #include <config.h>
 
@@ -421,10 +421,13 @@ int   main(int argc,char **argv)
    ProcWait::DeleteAll();
    DirColors::DeleteInstance();
    IdNameCacheCleanup();
+   SignalHook::Cleanup();
+   Log::Cleanup();
+   SMTask::Cleanup();
 
-   // the tasks left: SMTaskInit, LsCache::ExpireHelper, Log::global.
+   // the tasks left: LsCache::ExpireHelper, lftp_ssl_instance(if created)
    int task_count=SMTask::TaskCount();
-   if(task_count>3)
+   if(task_count>2)
       printf("WARNING: task_count=%d\n",task_count);
 
    return exit_code;
