@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp_rl.h,v 1.8 2002/09/03 13:16:35 lav Exp $ */
+/* $Id: lftp_rl.h,v 1.9 2004/11/03 08:37:44 lav Exp $ */
 
 #ifndef LFTP_RL_H
 #define LFTP_RL_H
@@ -52,7 +52,7 @@ void lftp_rl_init(
    const char *filename_quote_characters,
    char *(*filename_quoting_function)(char *,int,char *),
    char *(*filename_dequoting_function)(char *,int),
-   int (*char_is_quoted_p)(char *,int));
+   int (*char_is_quoted_p)(const char *,int));
 
 CDECL_END
 
