@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.68 2004/11/15 09:27:56 lav Exp $ */
+/* $Id: misc.cc,v 1.72 2005/04/28 22:59:39 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -440,8 +440,8 @@ int guess_year(int month,int day,int hour,int minute)
 {
    const struct tm &now=SMTask::now;
    int year=now.tm_year+1900;
-   if(((month     *32+        day)*64+       hour)*64+       minute
-    > ((now.tm_mon*32+now.tm_mday)*64+now.tm_hour)*64+now.tm_min+5)
+   if(month     *32+        day
+    > now.tm_mon*32+now.tm_mday+1)
       year--;
    return year;
 }
@@ -804,14 +804,20 @@ char *dirname_alloc(const char *fn)
    char *ret=xstrdup(fn);
    return dirname_modify(ret);
 }
-char *dirname_modify(char *ret)
+char *strip_trailing_slashes(char *ret)
 {
    int len=strlen(ret);
-
-   /* remove trailing slashes */
    while(len>0 && ret[len-1]=='/')
-      ret[--len] = 0;
-
+      len--;
+   if(len==0 && ret[0]=='/')
+      len=1+(ret[1]=='/');
+   if(len>0)
+      ret[len]=0;
+   return ret;
+}
+char *dirname_modify(char *ret)
+{
+   strip_trailing_slashes(ret);
    char *slash=strrchr(ret,'/');
    if(!slash)
       ret[0]=0; /* file with no path */
@@ -872,3 +878,35 @@ base64_encode (const char *s, char *store, int length)
   /* ...and zero-terminate it.  */
   *p = '\0';
 }
+
+bool temporary_network_error(int err)
+{
+   switch(err)
+   {
+   case(ETIMEDOUT):
+#ifdef ECONNRESET
+   case(ECONNRESET):
+#endif
+   case(ECONNREFUSED):
+#ifdef EHOSTUNREACH
+   case(EHOSTUNREACH):
+#endif
+#ifdef EHOSTDOWN
+   case(EHOSTDOWN):
+#endif
+#ifdef ENETRESET
+   case(ENETRESET):
+#endif
+#ifdef ENETUNREACH
+   case(ENETUNREACH):
+#endif
+#ifdef ENETDOWN
+   case(ENETDOWN):
+#endif
+#ifdef ECONNABORTED
+   case(ECONNABORTED):
+#endif
+      return true;
+   }
+   return false;
+}
