@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.h,v 1.32 2002/08/05 08:51:14 lav Exp $ */
+/* $Id: misc.h,v 1.37 2005/04/28 22:59:39 lav Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -110,5 +110,15 @@ char *xvasprintf(const char *format, va_list ap);
 char *xasprintf(const char *format, ...);
 
 char *dirname_alloc(const char *fn);
+char *dirname_modify(char *fn);
+char *strip_trailing_slashes(char *fn);
+
+/* returns last character of string or \0 if string is empty */
+char last_char(const char *str);
+
+int  base64_length (int len);
+void base64_encode (const char *s, char *store, int length);
+
+bool temporary_network_error(int e);
 
 #endif // MISC_H
