@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.110 2004/11/19 12:58:11 lav Exp $ */
+/* $Id: resource.cc,v 1.116 2005/05/18 05:57:10 lav Exp $ */
 
 #include <config.h>
 
@@ -144,7 +144,7 @@ const char *OrderValidate(char **s)
    return 0;
 }
 
-#ifdef USE_SSL
+#if USE_SSL
 static
 const char *AuthArgValidate(char **s)
 {
@@ -191,6 +191,7 @@ static ResDecl
    ResDecl01g("ftp:client",		  PACKAGE"/"VERSION,0,0),
    ResDecl01e("ftp:device-prefix",	  "no",	   ResMgr::BoolValidate,0),
    ResDecl01c("ftp:fix-pasv-address",	  "yes",   ResMgr::BoolValidate,0),
+   ResDecl01h("ftp:ignore-pasv-address",  "no",	   ResMgr::BoolValidate,0),
    ResDecl01d("ftp:fxp-force",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl02 ("ftp:fxp-passive-source",	  "no",	   ResMgr::BoolValidate,ResMgr::NoClosure),
    ResDecl02d("ftp:fxp-passive-sscn",	  "yes",   ResMgr::BoolValidate,ResMgr::NoClosure),
@@ -210,7 +211,7 @@ static ResDecl
    ResDecl09a("ftp:timezone",		  "GMT",   0,0),
    ResDecl10 ("ftp:skey-allow",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl11 ("ftp:skey-force",		  "no",    ResMgr::BoolValidate,0),
-#ifdef USE_SSL
+#if USE_SSL
    ResDecl11a("ftp:ssl-allow",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl11b("ftp:ssl-force",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl11c("ftp:ssl-protect-data",	  "no",	   ResMgr::BoolValidate,0),
@@ -250,6 +251,8 @@ static ResDecl
    ResDecl22 ("hftp:proxy",		  "",	   HttpProxyValidate,0),
    ResDecl22a("hftp:use-authorization",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl23 ("hftp:use-head",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl23c("hftp:use-mkcol",		  "no",	   ResMgr::BoolValidate,0),
+   ResDecl23b("hftp:use-propfind",	  "no",	   ResMgr::BoolValidate,0),
    ResDecl23a("hftp:use-type",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl24a("http:accept",		  "*/*",   0,0),
    ResDecl24b("http:accept-language",	  "",	   0,0),
@@ -258,6 +261,8 @@ static ResDecl
    ResDecl24 ("http:cache",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl24d("http:cache-control",	  "",	   0,0),
    ResDecl25 ("http:proxy",		  "",	   HttpProxyValidate,0),
+   ResDecl25a("http:use-mkcol",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl25b("http:use-propfind",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl26 ("http:user-agent",	  PACKAGE"/"VERSION,0,0),
    ResDecl27 ("http:cookie",		  "",	   0,0),
    ResDecl28 ("http:set-cookies",	  "no",	   0,0),
@@ -265,7 +270,7 @@ static ResDecl
    ResDecl29 ("http:put-method",	  "PUT",   PutOrPost,0),
    ResDecl30 ("http:put-content-type",	  "",	   0,0),
    ResDecl30b("http:referer",		  "",	   0,0),
-#ifdef USE_SSL
+#if USE_SSL
    ResDecl30a("https:proxy",		  "",	   HttpProxyValidate,0),
 #endif
    ResDecl31 ("net:idle",		  "180",   ResMgr::UNumberValidate,0),
@@ -292,28 +297,32 @@ static ResDecl
    ResDecl47 ("mirror:parallel-directories", "yes", ResMgr::BoolValidate,ResMgr::NoClosure),
    ResDecl48 ("mirror:parallel-transfer-count", "1", ResMgr::UNumberValidate,ResMgr::NoClosure),
    ResDecl49 ("mirror:exclude-regex",	  "(^|/)(\\.in\\.|\\.nfs)",ResMgr::ERegExpValidate,ResMgr::NoClosure),
-   ResDecl50 ("mirror:use-pget-n",	  "1",	   ResMgr::UNumberValidate,ResMgr::NoClosure);
+   ResDecl50 ("mirror:use-pget-n",	  "1",	   ResMgr::UNumberValidate,ResMgr::NoClosure),
+   ResDecl51 ("mirror:set-permissions",	  "yes",   ResMgr::BoolValidate,ResMgr::NoClosure);
 
 static ResDecl
    res_sftp_packets  ("sftp:max-packets-in-flight","16",ResMgr::UNumberValidate,0),
    res_sftp_proto_ver("sftp:protocol-version","4",ResMgr::UNumberValidate,0),
    res_sftp_rsize    ("sftp:size-read", "0x8000",ResMgr::UNumberValidate,0),
    res_sftp_wsize    ("sftp:size-write","0x8000",ResMgr::UNumberValidate,0),
-   res_sftp_conn_prog("sftp:connect-program","ssh -ax",0,0),
+   res_sftp_conn_prog("sftp:connect-program","ssh -a -x",0,0),
    res_sftp_srv_prog ("sftp:server-program","sftp",0,0);
 
 static ResDecl
    res_file_charset  ("file:charset",  "",   ResMgr::CharsetValidate,ResMgr::NoClosure);
 
-#ifdef USE_SSL
+#if USE_SSL
 static ResDecl
    res_ssl_ca_file    ("ssl:ca-file",  "", ResMgr::FileReadable,ResMgr::NoClosure),
-   res_ssl_ca_path    ("ssl:ca-path",  "", ResMgr::DirReadable, ResMgr::NoClosure),
    res_ssl_crl_file   ("ssl:crl-file", "", ResMgr::FileReadable,ResMgr::NoClosure),
-   res_ssl_crl_path   ("ssl:crl-path", "", ResMgr::DirReadable, ResMgr::NoClosure),
    res_ssl_key_file   ("ssl:key-file", "", ResMgr::FileReadable,0),
    res_ssl_cert_file  ("ssl:cert-file","", ResMgr::FileReadable,0),
    res_ssl_verify_cert("ssl:verify-certificate","no",ResMgr::BoolValidate,0);
+# if USE_OPENSSL
+static ResDecl
+   res_ssl_ca_path    ("ssl:ca-path",  "", ResMgr::DirReadable, ResMgr::NoClosure),
+   res_ssl_crl_path   ("ssl:crl-path", "", ResMgr::DirReadable, ResMgr::NoClosure);
+# endif
 #endif
 
 #if INET6
@@ -333,7 +342,7 @@ static ResDecl
 
 static ResDecl
    fish_shell  ("fish:shell", "/bin/sh",  0,0),
-   fish_connect("fish:connect-program", "ssh -ax", 0,0);
+   fish_connect("fish:connect-program", "ssh -a -x", 0,0);
 
 static ResDecl
    res_dir_colors ("color:dir-colors",   "",   0,ResMgr::NoClosure);
@@ -362,7 +371,7 @@ void ResMgr::ClassInit()
       Set("hftp:proxy",0,http_proxy);
    }
 
-#ifdef USE_SSL
+#if USE_SSL
    const char *https_proxy=getenv("https_proxy");
    if(https_proxy)
       Set("https:proxy",0,https_proxy);
